#!/usr/bin/python

import Tkinter
from Tkinter import *
import Tkinter as ttk
from os import listdir
from os.path import isfile, join
from tkFileDialog import askopenfilename
import time, sys, os, getopt
import struct, array
import serial
import uploader

BUTTONWIDTH = 10
PAD = 10

b = uploader.bootloader()
master = Tkinter.Tk()
master.title("PIC32 bootloader v0.2")
ports = b.serial_scan()
file_name = StringVar(master)
connection_label = StringVar(master)
info_label = StringVar(master)
dummyvar = StringVar(master)
reboot_type = StringVar(master)
destination_port = StringVar(master)

def select_port(port):
    destination_port.set(port)

def scan_ports():
    serial_ports = b.serial_scan()
    socket_ports = b.socket_scan()
    ports = serial_ports + socket_ports
 
    if ports:
        OptionMenu(com_group, dummyvar, *ports, command=select_port).grid(row=0, column=1, padx=PAD, pady=PAD)
        info_label.set( "Found %d serial port(s) and %d IP device(s). Check drop down list." % (len(serial_ports), len(socket_ports)) )
    else:
        info_label.set("No communication ports found.")

def select_file():
    file_name.set(askopenfilename())
    if b.select_file(file_name.get()):
        info_label.set("File \"%s\" loaded." % file_name.get())
    else:
        info_label.set("Problem with file!")

def flash():
    if not b.file_path:
        info_label.set("No valid file selected.")
        return

    # Erase old firmware
    info_label.set("Erasing...")
    master.update()
    for page in range (0, b.total_pages):     
        if not b.erase_page(page):
            info_label.set("Failed to erase page %d." % page)
            return

    # Program new firmware
    info_label.set("Writing (this may take a few minutes)... ")
    master.update()
    with open(b.file_path, 'rb') as f:
        for row in range (0, b.total_rows):       
            if not b.write_row(row, f.read(uploader.ROW_SIZE)):
                info_label.set("Failed to write row %d." % row)
                return

    info_label.set("Successfully wrote new firmware.")

def run_app():
    # Start new application
    info_label.set("Booting to application.")
    b.start_app()

def connect():
    if '.' in destination_port.get():
        b.socket_open(destination_port.get())
    else:
        b.serial_open(destination_port.get(), uploader.SERIAL_BAUD)

    # Connect to device
    if not (b.ser or b.target_ip):
        info_label.set("No port selected.")
        return

    info_label.set("Connecting...")
    master.update()    
        
    if b.connect(reboot_type.get()):
        connection_label.set(b.get_connection_data())
        info_label.set("Connected.")
    else:
        info_label.set("Failed to connect!")

# File
file_group = LabelFrame(master, text="File selection", padx=5, pady=5)
file_group.pack(expand="yes", fill="both", padx=10, pady=10, side=TOP)
Entry(file_group, textvariable=file_name, width=50).grid(column=0, row=0, padx=PAD, pady=PAD)
Button(file_group, text="Browse...", command=select_file, width=BUTTONWIDTH).grid(column=1, row=0, padx=PAD, pady=PAD)

# Communication
com_group = LabelFrame(master, text="Communication", padx=5, pady=5, width=100)
com_group.pack(expand="yes", fill="both", padx=10, pady=10, side=TOP, anchor=N)

Label(com_group, text="Select port or IP:", anchor=W).grid(row=0, column=0, sticky=W, padx=PAD, pady=PAD)
if ports:
    OptionMenu(com_group, dummyvar, *ports, command=select_port).grid(row=0, column=1, padx=PAD, pady=PAD)
else:
    info_label.set("No ports!")
Button(com_group, text="Scan again", command=scan_ports).grid(column=2, row=0, sticky=W, padx=PAD, pady=PAD)

Label(com_group, text="Selected destination:", anchor=W).grid(row=1, column=0, sticky=W, padx=PAD, pady=PAD)
Entry(com_group, textvariable=destination_port).grid(row=1, column=1, sticky=W, padx=PAD, pady=PAD)

# Reboot selection
Label(com_group, text="Reboot option (sent before connecting):", anchor=W).grid(row=2, column=0, sticky=W, padx=PAD, pady=PAD)
Radiobutton(com_group, text="Maritime Robotics Sensor Unit", variable=reboot_type, value='s').grid(row=3, column=0, sticky=W, padx=PAD)
Radiobutton(com_group, text="Skagmo Electronics Owl", variable=reboot_type, value='o').grid(row=4, column=0, sticky=W, padx=PAD)

# Author label
Label(master, text="Maritime Robotics and Skagmo Electronics, copyright 2015", anchor=W, justify=LEFT).pack(side=BOTTOM, pady=PAD)

# Progress
prog = LabelFrame(master, text="Information", padx=5, pady=5)
prog.pack(expand="yes", fill="both", padx=10, pady=10, side=BOTTOM)
Label(prog, textvariable=info_label, anchor=W, justify=LEFT).grid(row=0, column=0, padx=PAD, pady=PAD)

# Actions
actions = LabelFrame(master, text="Actions", padx=10, pady=10)
actions.pack(padx=10, pady=10, fill="both", side=LEFT)
Button(actions, text="Connect", command=connect, width=15).grid(column=0, row=0, padx=10, pady=15)
Button(actions, text="Flash", command=flash, width=15).grid(column=0, row=1, padx=10, pady=15)
Button(actions, text="Run app.", command=run_app, width=15).grid(column=0, row=2, padx=10, pady=15)

# Connection
conn = LabelFrame(master, text="Connection info", padx=5, pady=5, width=100)
conn.pack(expand="yes", fill="both", padx=10, pady=10, side=RIGHT, anchor=S)
Label(conn, textvariable=connection_label, anchor=W, justify=LEFT, font=("Courier", 10)).grid(row=1, column=0, sticky=W, padx=PAD, pady=PAD)

master.mainloop()

